package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.enums.KuaishouHourEnum;
import cn.com.ctop.job.kuaishou.enums.KuaishouMinSecEnum;
import cn.com.ctop.job.kuaishou.service.CommonService;
import cn.com.ctop.job.kuaishou.utils.Check;
import cn.com.ctop.job.kuaishou.utils.DateUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class JiaoYangFenXiaoJob {
    @Autowired
    private CommonService commonService;


    /**
     * 获取二创信息
     *
     * @throws Exception
     */
    @XxlJob("jyTopItem")
    public void jyTopItem() {
        try {
            String cookie = commonService.getCookieById(341063986L);
            if (Check.isNull(cookie)) {
                log.error("cookie数据为空");
                return;
            }
            String nowDate = DateUtils.getNowDateStr();
            Long statDate = Long.valueOf(nowDate.replace("-", ""));
            List<JSONObject> accountList = commonService.getAccountList();
            for (int i = 0; i < accountList.size(); i++) {
                JSONObject jsonObject = accountList.get(i);
                Long id = jsonObject.getLong("id");
                String name = jsonObject.getString("name");
                commonService.getItemTop(cookie, statDate, id, name, 0);
            }
        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
    }


    /**
     * 获取二创信息
     *
     * @throws Exception
     */
    @XxlJob("jyClipCooperation")
    public void jyClipCooperation() {
        try {
            List<JSONObject> cookies = commonService.getCookie();
            if (Check.isNull(cookies)) {
                log.error("cookie数据为空");
                return;
            }
            String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
            String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
            Long statDate = Long.valueOf(date.replace("-", ""));
            List<Integer> statusList = new ArrayList();

            statusList.add(1);

            for (int p = 0; p < cookies.size(); p++) {
                JSONObject cookieJson = cookies.get(p);
                String cookie = cookieJson.getString("cookie");
                for (int i = 0; i < statusList.size(); i++) {
                    Integer status = statusList.get(i);
                    commonService.getClipCooperation(cookie, 0, statDate, status);
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 获取二创信息
     *
     * @throws Exception
     */
    @XxlJob("jyClipCooperationV2")
    public void jyClipCooperationV2() {

        try {
            List<JSONObject> cookies = commonService.getCookie();
            if (Check.isNull(cookies)) {
                log.error("cookie数据为空");
                return;
            }
            String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
            String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
            Long statDate = Long.valueOf(date.replace("-", ""));
            List<Integer> statusList = new ArrayList();

            statusList.add(1);

            for (int p = 0; p < cookies.size(); p++) {
                JSONObject cookieJson = cookies.get(p);
                String cookie = cookieJson.getString("cookie");
                for (int i = 0; i < statusList.size(); i++) {
                    Integer status = statusList.get(i);
                    commonService.getClipCooperationV2(cookie, 0, statDate, status);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 当前小时数据获取
     **/
    @XxlJob("jyNowHourOrderGet")
    public void jyNowHourOrderGet() {
        try {
            log.info("-------------------------获取快分销订单当天当小时数据开始--------------------------");
            List<JSONObject> cookies = commonService.getCookie();
            if (Check.isNull(cookies)) {
                log.error("cookie数据为空");
                return;
            }
            String date = DateUtils.getNowDate("yyyy-MM-dd");
            Integer nowHour = DateUtils.getNowHour();
            JSONArray hours = new JSONArray();
            hours.add(nowHour);
            if (nowHour >= 1) {
                hours.add(nowHour - 1);
            }


            JSONArray minSecList = KuaishouMinSecEnum.getMinSecList();
            for (int i = 0; i < hours.size(); i++) {
                Integer hour = hours.getInteger(i);
                for (int j = 0; j < minSecList.size(); j++) {
                    JSONObject minSecJson = minSecList.getJSONObject(j);
                    String startTime = minSecJson.getString("startTime");
                    String endTime = minSecJson.getString("endTime");
                    String start = date + " " + hour + ":" + startTime;
                    String end = date + " " + hour + ":" + endTime;
                    System.err.println("开始时间：" + start);
                    System.err.println("结束时间：" + end);
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    Long orderCreateTimeStart = sdf.parse(start).getTime();
                    Long orderCreateTimeEnd = sdf.parse(end).getTime();
                    for (int p = 0; p < cookies.size(); p++) {
                        JSONObject cookieJson = cookies.get(p);
                        String cookie = cookieJson.getString("cookie");
                        commonService.getOrderList(cookie, 0, orderCreateTimeStart, orderCreateTimeEnd);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 当天前前小时数据获取
     **/
    /**
     * 当天数据获取
     **/
    @XxlJob("jyFrontHourOrderGet")
    public void frontHourOrderGet() {

        try {
            List<JSONObject> cookies = commonService.getCookie();
            if (Check.isNull(cookies)) {
                log.error("cookie数据为空");
                return;
            }
            String date = DateUtils.getNowDate("yyyy-MM-dd");
            JSONArray hourList = KuaishouHourEnum.getHourList();
            JSONArray minSecList = KuaishouMinSecEnum.getMinSecList();
            Integer nowHour = DateUtils.getNowHour();
            for (int i = 0; i < hourList.size(); i++) {
                String hour = hourList.getString(i);
                Integer integer = Integer.valueOf(hour.replace(":", "").replace(" ", ""));
                if (integer <= nowHour) {
                    for (int j = 0; j < minSecList.size(); j++) {
                        JSONObject minSecJson = minSecList.getJSONObject(j);
                        String startTime = minSecJson.getString("startTime");
                        String endTime = minSecJson.getString("endTime");
                        String start = date + hour + startTime;
                        String end = date + hour + endTime;
                        System.err.println("开始时间：" + start);
                        System.err.println("结束时间：" + end);
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        Long orderCreateTimeStart = sdf.parse(start).getTime();
                        Long orderCreateTimeEnd = sdf.parse(end).getTime();
                        for (int p = 0; p < cookies.size(); p++) {
                            JSONObject cookieJson = cookies.get(p);
                            String cookie = cookieJson.getString("cookie");
                            commonService.getOrderList(cookie, 0, orderCreateTimeStart, orderCreateTimeEnd);
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 昨日快分销数据获取
     **/
    @XxlJob("jyYesterdayOrderGet")
    public void jyYesterdayOrderGet() {
        try {
            log.info("-------------------------获取快分销订单昨日数据开始数据开始--------------------------");
            List<JSONObject> cookies = commonService.getCookie();
            if (Check.isNull(cookies)) {
                log.error("cookie数据为空");
                return;
            }
            String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
            String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
            JSONArray hourList = KuaishouHourEnum.getHourList();
            JSONArray minSecList = KuaishouMinSecEnum.getMinSecList();
            for (int i = 0; i < hourList.size(); i++) {
                String hour = hourList.getString(i);
                for (int j = 0; j < minSecList.size(); j++) {
                    JSONObject minSecJson = minSecList.getJSONObject(j);
                    String startTime = minSecJson.getString("startTime");
                    String endTime = minSecJson.getString("endTime");
                    String start = date + hour + startTime;
                    String end = date + hour + endTime;
                    System.err.println("开始时间：" + start);
                    System.err.println("结束时间：" + end);
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    Long orderCreateTimeStart = sdf.parse(start).getTime();
                    Long orderCreateTimeEnd = sdf.parse(end).getTime();
                    for (int p = 0; p < cookies.size(); p++) {
                        JSONObject cookieJson = cookies.get(p);
                        String cookie = cookieJson.getString("cookie");
                        commonService.getOrderList(cookie, 0, orderCreateTimeStart, orderCreateTimeEnd);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 前日日快分销数据获取
     **/
    @XxlJob("jyBeforeYesterdayOrderGet")
    public void jyBeforeYesterdayOrderGet() {
        try {
            log.info("-------------------------获取快分销订单前日数据开始数据开始--------------------------");
            List<JSONObject> cookies = commonService.getCookie();
            if (Check.isNull(cookies)) {
                log.error("cookie数据为空");
                return;
            }
            String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
            String date = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
            JSONArray hourList = KuaishouHourEnum.getHourList();
            JSONArray minSecList = KuaishouMinSecEnum.getMinSecList();
            for (int i = 0; i < hourList.size(); i++) {
                String hour = hourList.getString(i);
                for (int j = 0; j < minSecList.size(); j++) {
                    JSONObject minSecJson = minSecList.getJSONObject(j);
                    String startTime = minSecJson.getString("startTime");
                    String endTime = minSecJson.getString("endTime");
                    String start = date + hour + startTime;
                    String end = date + hour + endTime;
                    System.err.println("开始时间：" + start);
                    System.err.println("结束时间：" + end);
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    Long orderCreateTimeStart = sdf.parse(start).getTime();
                    Long orderCreateTimeEnd = sdf.parse(end).getTime();
                    for (int p = 0; p < cookies.size(); p++) {
                        JSONObject cookieJson = cookies.get(p);
                        String cookie = cookieJson.getString("cookie");
                        commonService.getOrderList(cookie, 0, orderCreateTimeStart, orderCreateTimeEnd);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 快分销-当前月所有数据
     **/
    @XxlJob("jyNowMonthyOrderGet")
    public void jyNowMonthyOrderGet() {
        try {
            List<JSONObject> cookies = commonService.getCookie();
            if (Check.isNull(cookies)) {
                log.error("cookie数据为空");
                return;
            }
            String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
            String endDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -3);
            String monthFirstDay = DateUtils.getMonthFirstDay();
            List<String> days = DateUtils.getDays(monthFirstDay, endDate);
            JSONArray hourList = KuaishouHourEnum.getHourList();
            JSONArray minSecList = KuaishouMinSecEnum.getMinSecList();
            for (int z = 0; z < days.size(); z++) {
                String date = days.get(z);
                for (int i = 0; i < hourList.size(); i++) {
                    String hour = hourList.getString(i);
                    for (int j = 0; j < minSecList.size(); j++) {
                        JSONObject minSecJson = minSecList.getJSONObject(j);
                        String startTime = minSecJson.getString("startTime");
                        String endTime = minSecJson.getString("endTime");
                        String start = date + hour + startTime;
                        String end = date + hour + endTime;
                        System.err.println("开始时间：" + start);
                        System.err.println("结束时间：" + end);
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        Long orderCreateTimeStart = sdf.parse(start).getTime();
                        Long orderCreateTimeEnd = sdf.parse(end).getTime();
                        for (int p = 0; p < cookies.size(); p++) {
                            JSONObject cookieJson = cookies.get(p);
                            String cookie = cookieJson.getString("cookie");
                            commonService.getOrderList(cookie, 0, orderCreateTimeStart, orderCreateTimeEnd);
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 上个月数据获取
     *
     * @throws Exception
     */
    @XxlJob("jyAllOrderGet")
    public void jyAllOrderGet() {
        try {
            //   List<String> dateList = new ArrayList<>();
            Map<String, String> monthStartAndEnd = DateUtils.getMonthStartAndEnd();
            String firstDay = monthStartAndEnd.get("firstDay");
            String lastDay = monthStartAndEnd.get("lastDay");
            List<String> days = DateUtils.getDays(firstDay, lastDay);
            List<JSONObject> cookies = commonService.getCookie();
            if (Check.isNull(cookies)) {
                log.error("cookie数据为空");
                return;
            }
            JSONArray hourList = KuaishouHourEnum.getHourList();
            JSONArray minSecList = KuaishouMinSecEnum.getMinSecList();
            for (int z = 0; z < days.size(); z++) {
                String date = days.get(z);
                for (int i = 0; i < hourList.size(); i++) {
                    String hour = hourList.getString(i);
                    for (int j = 0; j < minSecList.size(); j++) {
                        JSONObject minSecJson = minSecList.getJSONObject(j);
                        String startTime = minSecJson.getString("startTime");
                        String endTime = minSecJson.getString("endTime");
                        String start = date + hour + startTime;
                        String end = date + hour + endTime;
                        System.err.println("开始时间：" + start);
                        System.err.println("结束时间：" + end);
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                        Long orderCreateTimeStart = sdf.parse(start).getTime();
                        Long orderCreateTimeEnd = sdf.parse(end).getTime();
                        for (int p = 0; p < cookies.size(); p++) {
                            JSONObject cookieJson = cookies.get(p);
                            String cookie = cookieJson.getString("cookie");
                            commonService.getOrderList(cookie, 0, orderCreateTimeStart, orderCreateTimeEnd);
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
