package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.entity.CtopOauthToken;
import cn.com.ctop.job.kuaishou.entity.KuaishouAccountBalanceBudget;
import cn.com.ctop.job.kuaishou.service.CommonService;
import cn.com.ctop.job.kuaishou.service.TokenService;
import cn.com.ctop.job.kuaishou.utils.Check;
import cn.com.ctop.job.kuaishou.utils.DateUtils;
import com.alibaba.fastjson.JSONObject;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Component
public class KuaiShouBudgetJob {
    @Autowired
    private CommonService commonService;
    @Autowired
    private TokenService tokenService;

    static ExecutorService executorService = Executors.newFixedThreadPool(8);

    /**
     * 定时更新快手账户余额和预算
     */
    @XxlJob("kuaiShouBudgetJob")
    public void execute() {
        try {
            if (DateUtils.getNowHour() == 2) {
                //每日凌晨2点将预算全部置为无效，避免已关闭的账户留有僵尸数据。
                commonService.updateAllIsInvalidState();
            }
            List<CtopOauthToken> tokens = tokenService.getAllKuaiShouTokens();
            tokens.forEach(token -> executorService.submit(() -> {
                KuaishouAccountBalanceBudget budget = new KuaishouAccountBalanceBudget();
                JSONObject fundJson = commonService.fundGet(token);
                JSONObject accountBudget = commonService.getAccountBudget(token);
                if (!Check.isNull(accountBudget) && !Check.isNull(fundJson)) {
                    budget.setBalance(fundJson.getBigDecimal("balance"));
                    budget.setDayBudget(accountBudget.getBigDecimal("day_budget").divide(new BigDecimal(1000)).setScale(2, RoundingMode.HALF_UP));
                    budget.setDayBudgetSchedule(accountBudget.getString("day_budget_schedule"));
                    budget.setAccountId(token.getAccountId());
                    budget.setState(1);
                    commonService.replaceBudget(budget);
                }
            }));
            XxlJobHelper.log("定时更新快手账户余额和预算完成");
        } catch (ParseException e) {
            e.printStackTrace();
        }
    }
}

