package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.entity.CtopOauthToken;
import cn.com.ctop.job.kuaishou.service.AccountInfoService;
import cn.com.ctop.job.kuaishou.service.TokenService;
import cn.com.ctop.job.kuaishou.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 获取广告主分时报表
 *
 * @author syh
 */
@Component
public class KuaishouHourlyAccountReportLoadJob {
    @Autowired
    private AccountInfoService accountInfoService;
    @Autowired
    private TokenService tokenService;

    static ExecutorService executorService = Executors.newFixedThreadPool(8);

    /**
     * 快手广告主时报
     */
    @XxlJob("kuaishouHourlyAccountReport")
    public void execute() {
        try {
            Date getDate = new Date();
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH");
            String hour = simpleDateFormat.format(getDate);
            if ("00".equals(hour)) {
                getDate = DateUtils.addDay(getDate, -1);
            }
            //1:查询当日数据
            List<CtopOauthToken> tokens = tokenService.getAllKuaiShouTokens();
            if (null == tokens || tokens.size() <= 0) {
                XxlJobHelper.log("定时获取快手数据异常：未获取到可用的token");
                XxlJobHelper.handleFail();
                return;
            }
            Date finalGetDate = getDate;
            tokens.forEach(token -> executorService.submit(() -> accountInfoService.getAdvertiserReportHourly(token, finalGetDate, finalGetDate)));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
