package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.entity.CtopOauthToken;
import cn.com.ctop.job.kuaishou.service.AccountInfoService;
import cn.com.ctop.job.kuaishou.service.TokenService;
import cn.com.ctop.job.kuaishou.utils.DateUtils;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @author 于蒙
 */
@Component
public class KuaishouHourlyAccountWarningLoadJob {
    @Autowired
    private AccountInfoService accountInfoService;
    @Autowired
    private TokenService tokenService;
    static ExecutorService executorService = Executors.newFixedThreadPool(3);

    /**
     * 快手时段消耗预警
     */
    @XxlJob("kuaishouHourlyAccountWarning")
    public void execute() {
        try {
            XxlJobHelper.log("开始时段消耗预警定时任务");
            long l = System.currentTimeMillis();
            List<CtopOauthToken> tokens = tokenService.getAllKuaiShouTokens();
            if (null == tokens || tokens.size() <= 0) {
                XxlJobHelper.log("定时获取快手数据异常：未获取到可用的token");
                XxlJobHelper.handleFail();
                return ;
            }
            String statDate = DateUtils.getDate("yyyy-MM-dd");
            String anotherDay = DateUtils.getAnotherDay("yyyy-MM-dd", statDate, -1);
            tokens.forEach(token -> executorService.submit(() -> accountInfoService.kuaiShouAccountWarning(token.getAccountId(), statDate, anotherDay)));
            XxlJobHelper.log("异步时段消耗预警定时任务执行完毕，耗时：{} ms", System.currentTimeMillis() - l);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
