package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.service.RuixuanService;
import cn.com.ctop.job.kuaishou.service.TokenService;
import cn.com.ctop.job.kuaishou.utils.Check;
import cn.com.ctop.job.kuaishou.utils.DateUtils;
import cn.com.ctop.job.kuaishou.utils.HttpUtils;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class SettlementJob {

    @Autowired
    private RuixuanService ruixuanService;
    @Autowired
    private TokenService tokenService;


    @XxlJob("refreshCourierNumber")
    public void refreshCourierNumber() {
        try {
            String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
            String startDate = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -31);
            List<JSONObject> list = ruixuanService.getSamplesData(startDate);
            if (Check.isNull(list)) {
                return;
            }
            for (JSONObject jsonObject : list) {
                Long id = jsonObject.getLong("id");
                String courierNumber = jsonObject.getString("courierNumber");
                if (Check.isNull(courierNumber)) {
                    continue;
                }
                String companyCode = jsonObject.getString("companyCode");
                if (Check.isNull(companyCode)) {
                    continue;
                }
                Map<String, Object> map = new HashMap<>();
                map.put("id", id);
                map.put("courierNumber", courierNumber);
                map.put("companyCode", companyCode);
                HttpUtils.httpGet("http://ruixuan.api.tjyourong.com.cn/itemCollectSamples/refreshCourierNumber", map, null);
                Thread.sleep(1000L);
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }


    /**
     * 自定义创建监测链接预警
     */
    @XxlJob("cleanSettlementData")
    public void cleanSettlementData() {
        try {
            String url = "http://ruixuan.api.tjyourong.com.cn/settlement/data";
            String date = DateUtils.getLastDay(DateUtils.date2Str(), 1);
            Long statDate = Long.valueOf(date.replace("-", ""));
            List<JSONObject> list = ruixuanService.getDataList(statDate);
            if (Check.isNull(list)) {
                return;
            }
            List<List<JSONObject>> partition = Lists.partition(list, 1000);
            for (int i = 0; i < partition.size(); i++) {
                List<JSONObject> jsonObjects = partition.get(i);
                Map<String, Object> param = new HashMap<>();
                param.put("statDate", statDate);
                param.put("data", jsonObjects);
                HttpUtils.httpPostRequest(url, param, null);
            }
        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
    }


    @XxlJob("cleanPromoterBindChannel")
    public void cleanPromoterBindChannel() {
        try {
            Long date = Long.valueOf(DateUtils.getNowDate("yyyyMMdd"));
            ruixuanService.deletePromoterBindChannelByDate(date);
            ruixuanService.cleanKuaiShouPromoterBindChannel(date);
            ruixuanService.cleanBytedancePromoterBindChannel(date);
            ruixuanService.deleteMgsPromoterBindChannelByDate(date);
            ruixuanService.cleanMgsKuaiShouPromoterBindChannel(date);
        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
    }


}
