package cn.com.ctop.job.kuaishou.handler;

import cn.com.ctop.job.kuaishou.service.CommonService;
import cn.com.ctop.job.kuaishou.utils.DateUtils;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

@Component
@Slf4j
public class SynchronousAccountTransactionJob {

    @Autowired
    private CommonService commonService;

    /**
     * 同步头条账户交易流水
     *
     * @author ZZY
     */
    @XxlJob("synchronousAccountTransaction")
    public void execute() {
        try {
            SimpleDateFormat sd = new SimpleDateFormat("yyyy-MM-dd");
            Calendar ca = Calendar.getInstance();//得到一个Calendar的实例
            ca.setTime(new Date()); //设置时间为当前时间
            ca.add(Calendar.DAY_OF_MONTH, -1);  //设置为前一天
            String date = sd.format(ca.getTime());
            commonService.synchronousAccount();
            commonService.synchronousAccountTransactionDay(date, date);
            commonService.synchronousAccountTransactionDetails(date, date);
        } catch (Exception e) {
            log.info("同步完成");
        }
    }

    /**
     * 同步头条账户交易流水
     * 获取前3天数据
     */
    @XxlJob("synchronousAccountTransactionNew")
    public void synchronousAccountTransactionNew() {
        try {
            String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
            String yesterda1 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -1);
            String yesterda2 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -2);
            String yesterda3 = DateUtils.getAnotherDay("yyyy-MM-dd", nowDate, -3);
//            commonService.synchronousAccount();
            commonService.insertAccountTransactionDay(yesterda1, yesterda1);
            commonService.insertAccountTransactionDay(yesterda2, yesterda2);
            commonService.insertAccountTransactionDay(yesterda3, yesterda3);
        } catch (Exception e) {
            log.info("同步完成");
        }
    }
}
