package cn.com.ctop.job.kuaishou.service.impl;

import cn.com.ctop.job.kuaishou.entity.CtopOauthToken;
import cn.com.ctop.job.kuaishou.entity.EtlKuaishouReportAccountHourly;
import cn.com.ctop.job.kuaishou.entity.KuaiShouAppInfo;
import cn.com.ctop.job.kuaishou.entity.KuaiShouCampaign;
import cn.com.ctop.job.kuaishou.entity.KuaiShouDailyAgent;
import cn.com.ctop.job.kuaishou.entity.KuaiShouGroup;
import cn.com.ctop.job.kuaishou.entity.KuaiShouGroupTarget;
import cn.com.ctop.job.kuaishou.entity.KuaiShouSchedule;
import cn.com.ctop.job.kuaishou.entity.KuaishouReportHourlyAccount;
import cn.com.ctop.job.kuaishou.entity.OauthAgentToken;
import cn.com.ctop.job.kuaishou.entity.Project;
import cn.com.ctop.job.kuaishou.entity.UserAllocation;
import cn.com.ctop.job.kuaishou.mapper.AccountInfoMapper;
import cn.com.ctop.job.kuaishou.service.AccountInfoService;
import cn.com.ctop.job.kuaishou.service.ISendMessageService;
import cn.com.ctop.job.kuaishou.service.TokenService;
import cn.com.ctop.job.kuaishou.service.UserService;
import cn.com.ctop.job.kuaishou.utils.Check;
import cn.com.ctop.job.kuaishou.utils.DateUtils;
import cn.com.ctop.job.kuaishou.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxl.job.core.context.XxlJobHelper;
import lombok.extern.slf4j.Slf4j;
import lombok.var;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Service
public class AccountInfoServiceImpl implements AccountInfoService {
    @Autowired
    private AccountInfoMapper infoMapper;

    @Autowired
    private ISendMessageService sendMessageService;

    @Autowired
    private UserService userService;

    @Autowired
    private TokenService tokenService;

    @Override
    public void getGroupList(CtopOauthToken token, Date startDate, Date endDate) {
        getGroupListByPage(token, startDate, endDate, 1);
    }

    private void getGroupListByPage(CtopOauthToken token, Date startDate, Date endDate, int page) {
        //广告组信息
        String url = "https://ad.e.kuaishou.com/rest/openapi/v1/ad_unit/list";
        JSONObject param = new JSONObject();
        param.put("advertiser_id", token.getAccountId());
        param.put("page_size", 500);
        param.put("page", page);
        if (startDate != null && endDate != null) {
            param.put("start_date", DateUtils.formatDate(startDate));
            param.put("end_date", DateUtils.formatDate(endDate));
        }
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", token.getAccessToken());
        headers.put("Content-Type", "application/json");

        String result = HttpUtils.kuaiShouhttpPostRequest(url, param.toJSONString(), headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (Check.isNull(resultJson)) {
            log.error("获取广告组接口异常,advertiserId:{}", token.getAccountId());
            return;
        }
        Integer code = resultJson.getInteger("code");
        if (null == code || code != 0) {
            log.error("获取广告组返回结果异常,advertiserId:{},异常信息:{}", token.getAccountId(), resultJson);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (Check.isNull(details)) {
            return;
        }
        addGroup(token.getAccountId(), details);
        //   ruleKuaiShouPlanService.insertTarget(token.getAccountId(), details);
        getGroupListByPage(token, startDate, endDate, page + 1);
    }

    static ExecutorService executorBidTypeService = Executors.newFixedThreadPool(20);
    static ExecutorService executorMaxBidService = Executors.newFixedThreadPool(5);


    private void addGroup(Long accountId, JSONArray details) {
        JSONObject project = userService.getBidTypeAndMaxBid(accountId);
        if (Check.isNull(project) || Check.isNull(details)) {
            return;
        }
        // 最高出价
        Long maxBid = project.getLong("maxBid");

        //优化目标集
        List<Integer> bidList = null;
        String bidTypeStr = project.getString("bidType");
        if (!Check.isNull(bidTypeStr)) {
            bidList = JSONArray.parseArray(bidTypeStr, Integer.class);
        }

        //转化目标map
        Map<Integer, Long> ocpxMap = new HashMap<>();
        String ocpxTypeStr = project.getString("ocpxActionType");
        if (!Check.isNull(ocpxTypeStr)) {
            JSONArray ocpxArr = JSONArray.parseArray(ocpxTypeStr);
            for (int i = 0; i < ocpxArr.size(); i++) {
                JSONObject obj = ocpxArr.getJSONObject(i);
                ocpxMap.put(obj.getInteger("ocpxCode"), obj.getLong("maxValue"));
            }
        }

        //深度转化目标map
        Map<Integer, Long> deepMap = new HashMap<>();
        String deepTypeStr = project.getString("deepConversionType");
        if (!Check.isNull(deepTypeStr)) {
            JSONArray deepArr = JSONArray.parseArray(deepTypeStr);
            for (int i = 0; i < deepArr.size(); i++) {
                JSONObject obj = deepArr.getJSONObject(i);
                ocpxMap.put(obj.getInteger("deepCode"), obj.getLong("deepValue"));
            }
        }
        List<KuaiShouGroup> groups = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject detail = JSONObject.parseObject(details.get(i).toString());
            if (!Check.isNull(detail)) {
                //优化目标
                Integer bidType = detail.getInteger("bid_type");
                //优化出价（只有单击时，才有值）
                Long bid = detail.getLong("bid");
                //转化目标
                Integer ocpxActionType = detail.getInteger("ocpx_action_type");
                //转化目标出价
                Long cpaBid = detail.getLong("cpa_bid");
                //深度转化目标
                Integer deepConversionType = detail.getInteger("deep_conversion_type");
                //深度转化目标出价
                Long deepConversionBid = detail.getLong("deep_conversion_bid");
                //单日预算
                Long dayBudget = detail.getLong("day_budget");
                //组ID
                Long unitId = detail.getLong("unit_id");
                //组名称
                String unitName = detail.getString("unit_name");
                //状态
                Integer status = detail.getInteger("status");
                //投放状态
                Integer putStatus = detail.getInteger("put_status");
                if (putStatus == 1 && status != 15) { // 组状态为 投放中的 and 广告组状态为非暂停
                    Boolean typeFalg = true;
                    Boolean bidFalg = true;

                    //优化目标
                    if (!bidList.isEmpty()) {
                        if (!bidList.contains(bidType)) {
                            //项目不包含该优化目标，则关停
                            typeFalg = false;
                        } else if (bidType == 2 && bid > maxBid) {
                            //若优化目标为2点击数，并且广告组出价大于项目设置最大出价，则关停
                            bidFalg = false;
                        }
                    }
                    //转化目标
                    if (!Check.isNull(ocpxActionType) && ocpxActionType != 0) {
                        //！=0是设置过转化目标
                        if (!Check.isNullMap(ocpxMap)) {
                            Long ocpxBid = ocpxMap.get(ocpxActionType);
                            if (Check.isNull(ocpxBid)) {
                                if (ocpxActionType == 2 && dayBudget != 0 && dayBudget <= 500 * 1000) {
                                    // 转化目标为2行为数，并且预算小于等于500且不是不限，则允许启动
                                    typeFalg = true;
                                } else {
                                    //通过key未获取值，说明不包含该目标，进行关停
                                    typeFalg = false;
                                }
                            } else {
                                if (cpaBid > ocpxBid) {
                                    //组转化出价 大于 项目设置最大转化出价，则关停
                                    bidFalg = false;
                                }
                            }
                        } else if (ocpxActionType == 2 && (dayBudget == 0 || dayBudget > 500 * 1000)) {
                            // 转化目标为2行为数，并且预算大于500 或者不限，则关停
                            typeFalg = false;
                        }
                    }

                    //转化目标
                    if (!Check.isNull(deepConversionType) && deepConversionType != 0 && !deepMap.isEmpty()) {
                        Long deepBid = deepMap.get(deepConversionType);
                        if (Check.isNull(deepBid)) {
                            //通过key未获取值，说明不包含该目标，进行关停
                            typeFalg = false;
                        } else {
                            if (deepConversionBid > deepBid) {
                                //组深度转化出价 大于 项目设置最大转化出价，则关停
                                bidFalg = false;
                            }
                        }
                    }

                    if (!typeFalg) { // 出价方式设置非法
                        executorBidTypeService.submit(new Runnable() {
                            @Override
                            public void run() {
                                sendMessageService.kuaiShouWarningOperation(project.getLong("projectId"), project.getString("projectName"), accountId, unitId, unitName);
                            }
                        });
                    }
                    if (!bidFalg) {
                        executorMaxBidService.submit(new Runnable() {
                            @Override
                            public void run() {
                                sendMessageService.kuaiShouBidWarningOperation(project.getLong("projectId"), project.getString("projectName"), accountId, unitId, unitName);
                            }
                        });
                    }
                }
                KuaiShouGroup group = JSONObject.parseObject(detail.toJSONString(), KuaiShouGroup.class);
                group.setId("" + accountId + unitId);
                group.setAccountId(accountId);
                group.setGroupCreateTime(detail.getString("create_time"));
                group.setGroupUpdateTime(detail.getString("update_time"));
                group.setUnitSource(detail.getInteger("unit_source"));
                group.setCreateTime(new Date());
                group.setUpdateTime(new Date());
                groups.add(group);
            }
        }
        if (!Check.isNull(groups)) {
            try {
                Thread.sleep(200L);
                infoMapper.replaceBatchGroup(groups);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void updateUnitStatus(String token, Long advertiserId, Long unitId, int putStatus, String loginId) {
        try {
            //修改广告组状态
            String url = "https://ad.e.kuaishou.com/rest/openapi/v1/ad_unit/update/status";
            Map<String, String> header = new HashMap<>();
            header.put("Access-Token", token);
            header.put("Content-Type", "application/json");
            JSONObject params = new JSONObject();
            params.put("advertiser_id", advertiserId);
            params.put("unit_id", unitId);
            params.put("put_status", putStatus);
            String result = HttpUtils.httpPostRequest(url, params, header);
            JSONObject resultJson = JSONObject.parseObject(result);
            if (!Check.isNull(resultJson)) {
                Integer code = resultJson.getInteger("code");
                if (code == 0) {
                    if (putStatus == 3) {
                        infoMapper.deleteUnitById(advertiserId, unitId);
                    } else {
                        getGroupByUnitId(token, advertiserId, unitId);
                    }
                    log.info("删除成功，accountId：{}。unitId：{}", advertiserId, unitId);
                } else {
                    log.error("修改广告组状态失败，advertiserId：{}，unitId：{}，返回信息：{}", advertiserId, unitId, resultJson);
                }
            } else {
                log.error("修改广告组状态返回结果为空，advertiserId：{}，unitId：{}", advertiserId, unitId);
            }
        } catch (Exception e) {
            log.error("修改广告组状态异常，advertiserId：{}，unitId：{}", advertiserId, unitId);
            e.printStackTrace();
        }
    }

    public void getGroupByUnitId(String accessToken, Long accountId, Long unitId) {
        //广告组信息
        String url = "https://ad.e.kuaishou.com/rest/openapi/v1/ad_unit/list";
        JSONObject param = new JSONObject();
        param.put("advertiser_id", accountId);
        param.put("unit_id", unitId);
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        headers.put("Content-Type", "application/json");

        String result = HttpUtils.kuaiShouhttpPostRequest(url, param.toJSONString(), headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (Check.isNull(resultJson)) {
            log.error("获取广告组接口异常,advertiserId:{}", accountId);
            return;
        }
        Integer code = resultJson.getInteger("code");
        if (null == code || code != 0) {
            log.error("获取广告组返回结果异常,advertiserId:{},异常信息:{}", accountId, resultJson);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (Check.isNull(details)) {
            return;
        }
        addGroupV2(accountId, details);
        log.info("组数据入库成功=》id：{}", unitId);
    }


    private void addGroupV2(Long advertiserId, JSONArray details) {
        JSONObject json = userService.getBidTypeAndMaxBid(advertiserId);
        Long maxBid = json.getLong("maxBid");// 最高出价
        String bidTypeStr = json.getString("bidType"); // 出价方式
        JSONArray bidTypeArr = JSONArray.parseArray(bidTypeStr);
        String ocpxActionTypeStr = json.getString("ocpxActionType"); // 优化目标
        JSONArray ocpxArr = JSONArray.parseArray(ocpxActionTypeStr);
        if (!Check.isNull(details)) {
            List<KuaiShouGroup> groups = new ArrayList<>();
            List<KuaiShouAppInfo> appInfos = new ArrayList<>();
            List<KuaiShouSchedule> schedules = new ArrayList<>();
            List<KuaiShouGroupTarget> targets = new ArrayList<>();
            for (int i = 0; i < details.size(); i++) {
                JSONObject detail = JSONObject.parseObject(details.get(i).toString());
                if (!Check.isNull(detail)) {
                    Integer bid_type = detail.getInteger("bid_type");
                    Long bid = detail.getLong("bid");
                    Long cpa_bid = detail.getLong("cpa_bid");
                    Long day_budget = detail.getLong("day_budget");
                    Integer ocpx_action_type = detail.getInteger("ocpx_action_type");
                    Long unitId = detail.getLong("unit_id");
                    String unit_name = detail.getString("unit_name");
                    Integer status = detail.getInteger("status");
                    Integer put_status = detail.getInteger("put_status");
                    if (put_status == 1) { // 组状态为 投放中的
                        if (status != 15) { // 广告组状态为非暂停
                            Boolean bidTypeTrueOrFalse = false;
                            for (int j = 0; j < bidTypeArr.size(); j++) {
                                Integer bidType = (Integer) bidTypeArr.get(j);
                                if (bidType.equals(bid_type)) {
                                    bidTypeTrueOrFalse = true;
                                    break;
                                }
                            }

                            Boolean ocpxTrueOrFalse = false;
                            if (ocpx_action_type == 0) {
                                ocpxTrueOrFalse = true;
                            } else {
                                for (int j = 0; j < ocpxArr.size(); j++) {
                                    Integer ocpxActionType = (Integer) ocpxArr.get(j);
                                    if (ocpxActionType.equals(ocpx_action_type)) {
                                        ocpxTrueOrFalse = true;
                                        break;
                                    }
                                }
                            }


                            if (!bidTypeTrueOrFalse || !ocpxTrueOrFalse) { // 出价方式设置非法
                                if (day_budget > 500 * 1000 || day_budget == 0) { // 预算大于500 或者不限制预算 需要发送通知 并且进行暂停操作
                                    executorBidTypeService.submit(new Runnable() {
                                        @Override
                                        public void run() {
                                            sendMessageService.kuaiShouWarningOperation(json.getLong("projectId"), json.getString("projectName"), advertiserId, unitId, unit_name);
                                        }
                                    });
                                }
                            }
                            if (bid > maxBid || cpa_bid > maxBid) {
                                executorMaxBidService.submit(new Runnable() {
                                    @Override
                                    public void run() {
                                        sendMessageService.kuaiShouBidWarningOperation(json.getLong("projectId"), json.getString("projectName"), advertiserId, unitId, unit_name);
                                    }
                                });
                            }
                        }
                    }

                    KuaiShouGroup group = new KuaiShouGroup();
                    group.setId("" + advertiserId + unitId);
                    group.setAccountId(advertiserId);
                    group.setCampaignId(detail.getLong("campaign_id"));
                    group.setUnitId(unitId);
                    group.setUnitName(unit_name);
                    group.setStatus(status);
                    group.setPutStatus(put_status);
                    group.setCreateChannel(detail.getInteger("create_channel"));
                    group.setReviewDetail(detail.getString("review_detail"));
                    group.setBidType(bid_type);
                    group.setBid(bid);
                    group.setCpaBid(detail.getLong("cpa_bid"));
                    group.setOcpxActionType(ocpx_action_type);
                    group.setDeepConversionType(detail.getInteger("deep_conversion_type"));
                    group.setDeepConversionBid(detail.getLong("deep_conversion_bid"));
                    group.setDayBudget(detail.getLong("day_budget"));
                    group.setSpeed(detail.getInteger("speed"));
                    group.setBeginTime(detail.getString("begin_time"));
                    group.setEndTime(detail.getString("end_time"));
                    group.setScheduleTime(detail.getString("schedule_time"));
                    group.setSceneId(detail.getJSONArray("scene_id") + "");
                    group.setShowMode(detail.getInteger("show_mode"));
                    group.setUnitType(detail.getInteger("unit_type"));
                    group.setUrlType(detail.getInteger("url_type"));
                    group.setUrl(detail.getString("url"));
                    group.setSchemaUri(detail.getString("schema_uri"));
                    group.setAppId(detail.getLong("app_id"));
                    group.setAppIconUrl(detail.getString("app_icon_url"));
                    group.setGroupCreateTime(detail.getString("create_time"));
                    group.setGroupUpdateTime(detail.getString("update_time"));
                    group.setConvertId(detail.getLong("convert_id"));
                    group.setUseAppMarket(detail.getInteger("use_app_market"));
                    group.setAppStore(detail.getJSONArray("app_store") + "");
                    group.setCreateTime(new Date());
                    group.setUpdateTime(new Date());
                    groups.add(group);
                    // 添加应用信息
                    JSONObject diverseJson = detail.getJSONObject("diverse_data");
                    if (!Check.isNull(diverseJson)) {
                        KuaiShouAppInfo appInfo = new KuaiShouAppInfo();
                        appInfo.setAccountId(advertiserId);
                        appInfo.setAppName(diverseJson.getString("app_name"));
                        appInfo.setAppPackageName(diverseJson.getString("app_package_name"));
                        appInfo.setDeviceOsType(diverseJson.getInteger("device_os_type"));
                        appInfo.setUnitId(unitId);
                        appInfo.setCreateTime(new Date());
                        appInfo.setUpdateTime(new Date());
                        appInfos.add(appInfo);
                    }
                    //时间段信息
                    JSONObject scheduleJson = detail.getJSONObject("schedule");
                    if (!Check.isNull(scheduleJson)) {
                        KuaiShouSchedule schedule = new KuaiShouSchedule();
                        schedule.setUnitId(unitId);
                        schedule.setAccountId(advertiserId);
                        schedule.setMon(scheduleJson.getJSONArray("mon").toJSONString());
                        schedule.setTues(scheduleJson.getJSONArray("tues").toJSONString());
                        schedule.setWed(scheduleJson.getJSONArray("wed").toJSONString());
                        schedule.setThur(scheduleJson.getJSONArray("thur").toJSONString());
                        schedule.setFri(scheduleJson.getJSONArray("fri").toJSONString());
                        schedule.setSat(scheduleJson.getJSONArray("sat").toJSONString());
                        schedule.setSun(scheduleJson.getJSONArray("sun").toJSONString());
                        schedule.setCreateTime(new Date());
                        schedule.setUpdateTime(new Date());
                        schedules.add(schedule);
                    }

                    JSONObject targetJson = detail.getJSONObject("target");
                    if (!Check.isNull(targetJson)) {
                        KuaiShouGroupTarget groupTarget = new KuaiShouGroupTarget();
                        groupTarget.setAccountId(advertiserId);
                        groupTarget.setUnitId(unitId);
                        if (!Check.isNull(targetJson.getJSONArray("region"))) {
                            groupTarget.setRegion(targetJson.getJSONArray("region").toJSONString());
                        }

                        JSONObject age = targetJson.getJSONObject("age");
                        if (!Check.isNull(age)) {
                            groupTarget.setAgeMin(age.getInteger("min"));
                            groupTarget.setAgeMax(age.getInteger("max"));
                        }
                        if (!Check.isNull(targetJson.getJSONArray("ages_range"))) {
                            groupTarget.setAgesRange(targetJson.getJSONArray("ages_range").toJSONString());
                        }

                        groupTarget.setGender(targetJson.getInteger("gender"));
                        groupTarget.setPlatformOs(targetJson.getInteger("platform_os"));
                        groupTarget.setAndroidOsv(targetJson.getInteger("android_osv"));
                        groupTarget.setIosOsv(targetJson.getInteger("ios_osv"));
                        groupTarget.setNetwork(targetJson.getInteger("network"));
                        if (!Check.isNull(targetJson.getJSONArray("device_brand"))) {
                            groupTarget.setDeviceBrand(targetJson.getJSONArray("device_brand").toJSONString());
                        }

                        if (!Check.isNull(targetJson.getJSONArray("device_price"))) {
                            groupTarget.setDevicePrice(targetJson.getJSONArray("device_price").toJSONString());
                        }

                        groupTarget.setBusinessInterestType(targetJson.getInteger("business_interest_type"));
                        if (!Check.isNull(targetJson.getJSONArray("business_interest"))) {
                            groupTarget.setBusinessInterest(targetJson.getJSONArray("business_interest").toJSONString());
                        }

                        if (!Check.isNull(targetJson.getJSONArray("fans_star"))) {
                            groupTarget.setFansStar(targetJson.getJSONArray("fans_star").toJSONString());
                        }

                        if (!Check.isNull(targetJson.getJSONArray("interest_video"))) {
                            groupTarget.setInterestVideo(targetJson.getJSONArray("interest_video").toJSONString());
                        }

                        if (!Check.isNull(targetJson.getJSONArray("app_interest"))) {
                            groupTarget.setAppInterest(targetJson.getJSONArray("app_interest").toJSONString());
                        }

                        if (!Check.isNull(targetJson.getJSONArray("app_ids"))) {
                            groupTarget.setAppIds(targetJson.getJSONArray("app_ids").toJSONString());
                        }


                        if (!Check.isNull(targetJson.getInteger("filter_converted_level"))) {
                            groupTarget.setFilterConvertedLevel(targetJson.getInteger("filter_converted_level"));
                        }


                        if (!Check.isNull(targetJson.getJSONArray("population"))) {
                            groupTarget.setPopulation(targetJson.getJSONArray("population").toJSONString());
                        }

                        if (!Check.isNull(targetJson.getJSONArray("exclude_population"))) {
                            groupTarget.setExcludePopulation(targetJson.getJSONArray("exclude_population").toJSONString());
                        }

                        if (!Check.isNull(targetJson.getJSONArray("third_platform_code"))) {
                            groupTarget.setThirdPlatformCode(targetJson.getJSONArray("third_platform_code").toJSONString());
                        }


                        JSONObject intelliExtend = targetJson.getJSONObject("intelli_extend");
                        if (!Check.isNull(intelliExtend)) {
                            groupTarget.setIsOpen(intelliExtend.getInteger("is_open"));
                            groupTarget.setNoAgeBreak(intelliExtend.getInteger("no_age_break"));
                            groupTarget.setNoGenderBreak(intelliExtend.getInteger("no_gender_break"));
                            groupTarget.setNoAreaBreak(intelliExtend.getInteger("no_area_break"));

                        }
                        //行为兴趣定向
                        JSONObject behaviorInterest = targetJson.getJSONObject("behavior_interest");
                        if (!Check.isNull(behaviorInterest)) {
                            JSONObject behavior = behaviorInterest.getJSONObject("behavior");
                            if (!Check.isNull(behavior)) {
                                if (!Check.isNull(behavior.getJSONArray("keyword"))) {
                                    groupTarget.setKeyword(JSONObject.toJSONString(behavior.getJSONArray("keyword")));
                                }
                                if (!Check.isNull(behavior.getString("label"))) {
                                    groupTarget.setLabel(behavior.getString("label"));
                                }
                                if (!Check.isNull(behavior.getInteger("time_type"))) {
                                    groupTarget.setTimeType(behavior.getInteger("time_type"));
                                }
                                if (!Check.isNull(behavior.getInteger("strength_type"))) {
                                    groupTarget.setStrengthType(behavior.getInteger("strength_type"));
                                }
                                if (!Check.isNull(behavior.getString("scene_type"))) {
                                    groupTarget.setSceneType(behavior.getString("scene_type"));
                                }
                            }
                            JSONObject interest = behaviorInterest.getJSONObject("interest");
                            if (!Check.isNull(interest)) {
                                if (!Check.isNull(interest.getString("label"))) {
                                    groupTarget.setInterestLabel(interest.getString("label"));
                                }
                            }
                        }
                        targets.add(groupTarget);
                    }
                }
            }
            if (!Check.isNull(groups)) {
                infoMapper.replaceBatchGroup(groups);
            }
            if (!Check.isNull(schedules)) {
                infoMapper.replaceBatchShouSchedule(schedules);
            }
            if (!Check.isNull(appInfos)) {
                infoMapper.replaceBatchAppInfo(appInfos);
            }
            if (!Check.isNull(targets)) {
                infoMapper.replaceBatchGroupTarget(targets);
            }
        }
    }

    @Override
    public void queryEveryhourlyDataForAccount() {
        List<EtlKuaishouReportAccountHourly> list = infoMapper.queryEveryhourlyDataForAccount();
        if (!Check.isNull(list)) {
            infoMapper.replaceBatchEtlHourlyAccount(list);
        }
        infoMapper.replaceBatchEtlDailyAccount(list);
    }

    @Override
    public void getAdvertiserReportHourly(CtopOauthToken token, Date startDate, Date endDate) {
        try {
            Thread.sleep(200L);
            getAdvertiserReportHourlyByPage(token, startDate, endDate, 1);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
//kuaishou_api_url=https://ad.e.kuaishou.com

    private void getAdvertiserReportHourlyByPage(CtopOauthToken token, Date startDate, Date endDate, int page) {
        String url = "https://ad.e.kuaishou.com/rest/openapi/v1/report/account_report";
        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", token.getAccessToken());
        Map<String, Object> param = new HashMap<>();
        param.put("start_date", DateUtils.formatDate(startDate));
        param.put("end_date", DateUtils.formatDate(endDate));
        param.put("advertiser_id", token.getAccountId());
        param.put("temporal_granularity", "HOURLY");
        param.put("page_size", 2000);
        param.put("page", page);
        String result = HttpUtils.httpPostRequest(url, param, headers);
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        JSONObject resultJson = JSONObject.parseObject(result);
        Integer code = resultJson.getInteger("code");
        String message = resultJson.getString("message");
        if (null == code || code != 0) {
            log.error("获取快手广告主时报表异常:{},accountId:{}", message, token.getAccountId());
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (null == details || details.size() <= 0) {
            log.error("快手时报返回详情为空，accountId：{}", token.getAccountId());
            return;
        }
        List<KuaishouReportHourlyAccount> addList = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            JSONObject detailJson = details.getJSONObject(i);
            var show = detailJson.getLong("show");
            detailJson.put("photo_show", show);
            var like = detailJson.getLong("like");
            detailJson.put("photo_like", like);
            KuaishouReportHourlyAccount kuaishouReportHourlyAccount = JSONObject.toJavaObject(detailJson, KuaishouReportHourlyAccount.class);
            kuaishouReportHourlyAccount.setAccountId(token.getAccountId());
            kuaishouReportHourlyAccount.setId("" + token.getAccountId() + kuaishouReportHourlyAccount.getStatDate() + kuaishouReportHourlyAccount.getStatHour());
            //  hourlyAccountService.saveOrUpdate(kuaishouReportHourlyAccount);
            addList.add(kuaishouReportHourlyAccount);
        }
        infoMapper.replaceBatchKuaishouReportHourlyAccount(addList);
        log.info("获取账户时报数据完成，accountId：{}", token.getAccountId());
    }

    /**
     * 快手账户 出价预警
     *
     * @param accountId
     */
    @Override
    public void kuaiShouAccountWarning(Long accountId, String statDate, String anotherDay) {
        try {
            Integer maxHour = infoMapper.selectMaxHour(accountId, statDate);
            if (Check.isNull(maxHour)) {
                return;
            }
            UserAllocation userAllocation = userService.getOneByAccountId(accountId);
            if (Check.isNull(userAllocation)) {
                return;
            }
            if (userAllocation.getAccountStatus() == 1) {
                return;
            }
            Project project = userService.getProjectById(userAllocation.getProjectId());
            BigDecimal warningAmount = userAllocation.getWarningAmount();
            BigDecimal warningProportion = userAllocation.getWarningProportion();
            BigDecimal nowCost = infoMapper.getkuaishouReportHourlyCost(accountId, maxHour, statDate);
            if (maxHour != 0) {
                if (!Check.isNull(nowCost) && nowCost.compareTo(new BigDecimal(0)) != 0) {
                    BigDecimal lastHourCost = infoMapper.getkuaishouReportHourlyCost(accountId, maxHour - 1, statDate);
                    if (!Check.isNull(lastHourCost) && lastHourCost.compareTo(new BigDecimal(0)) != 0) {
                        BigDecimal subtract = nowCost.subtract(lastHourCost);
                        BigDecimal divide = nowCost.divide(lastHourCost, 2, RoundingMode.HALF_UP);
                        if (subtract.compareTo(warningAmount) > -1 && divide.compareTo(warningProportion) > -1) {
                            sendMessageService.sendChargeEarlyWarningMessage(accountId, userAllocation.getAuthName(), maxHour, nowCost, userAllocation.getProjectId(), project.getProjectName(), userAllocation.getUserName());
                        }
                    }
                }
            }
            if (!Check.isNull(nowCost) && nowCost.compareTo(new BigDecimal(0)) != 0) {
                BigDecimal yesterdayHourCost = infoMapper.getkuaishouReportHourlyCost(accountId, maxHour, anotherDay);
                if (!Check.isNull(yesterdayHourCost) && yesterdayHourCost.compareTo(new BigDecimal(0)) != 0) {
                    BigDecimal subtract = nowCost.subtract(yesterdayHourCost);
                    BigDecimal divide = nowCost.divide(yesterdayHourCost, 2, RoundingMode.HALF_UP);
                    if (subtract.compareTo(warningAmount) > -1 && divide.compareTo(warningProportion) > -1) {
                        sendMessageService.sendYesterdayChargeEarlyWarningMessageMessage(accountId, userAllocation.getAuthName(), maxHour, nowCost, userAllocation.getProjectId(), project.getProjectName(), userAllocation.getUserName());
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void getAgentReportByPage(String startDate, String endDate) {
        List<Long> appList = new ArrayList<>();
        appList.add(27L);
        appList.add(165893351L);
        for (int i = 0; i < appList.size(); i++) {
            Long appId = appList.get(i);
            log.info("获取代理商数据AppId：{}", appId);
            OauthAgentToken agentToken = tokenService.getAgentTokenByAppId(appId);
            getAgentReport(agentToken.getAccessToken(), agentToken.getAgentId(), startDate, endDate, 1);
        }
    }


    private void getAgentReport(String token, Long agentId, String startDate, String endDate, int page) {
        try {
            String url = "https://ad.e.kuaishou.com/rest/openapi/v1/agent/report";
            Map<String, String> headers = new HashMap<String, String>();
            headers.put("Content-Type", " application/json");
            headers.put("Access-Token", token);
            JSONObject param = new JSONObject();
            param.put("start_date", startDate);
            param.put("end_date", endDate);
            param.put("agent_id", agentId);
            param.put("page_size", 1000);
            param.put("page", page);

            String result = HttpUtils.kuaiShouhttpPostRequest(url, param.toJSONString(), headers);
            JSONObject resultJson = JSONObject.parseObject(result);
            if (Check.isNull(resultJson)) {
                log.info("代理商数据返回为空");
                return;
            }
            Integer code = resultJson.getInteger("code");
            if (code != 0) {
                log.error("代理商报表返回code错误，返回：{}", resultJson);
                return;
            }
            JSONObject data = resultJson.getJSONObject("data");
            if (Check.isNull(data)) {
                log.error("代理商报表返回data节点为空");
                return;
            }
            JSONArray details = data.getJSONArray("details");
            if (Check.isNull(details)) {
                log.error("代理商报表返回details节点为空");
                return;
            }
            List<KuaiShouDailyAgent> agentList = new ArrayList<>();
            for (int i = 0; i < details.size(); i++) {
                JSONObject detailJson = details.getJSONObject(i);
                if (!Check.isNull(detailJson)) {
                    KuaiShouDailyAgent agent = new KuaiShouDailyAgent();
                    agent.setAgentId(agentId);
                    agent.setAccountId(detailJson.getLong("account_id"));
                    agent.setDateTime(detailJson.getString("date_time"));
                    agent.setUserId(detailJson.getLong("user_id"));
                    agent.setAccountName(detailJson.getString("account_name"));
                    agent.setTotalChargedInYuan(detailJson.getBigDecimal("total_charged_in_yuan"));
                    agent.setTotalBalanceInYuan(detailJson.getBigDecimal("total_balance_in_yuan"));
                    agent.setRealChargedInYuan(detailJson.getBigDecimal("real_charged_in_yuan"));
                    agent.setTotalRebateRealChargedInYuan(detailJson.getBigDecimal("total_rebate_real_charged_in_yuan"));
                    agent.setContractRebateRealChargedInYuan(detailJson.getBigDecimal("contract_rebate_real_charged_in_yuan"));
                    agent.setDirectRebateRealChargedInYuan(detailJson.getBigDecimal("direct_rebate_real_charged_in_yuan"));
                    agent.setCreditRealChargedInYuan(detailJson.getBigDecimal("credit_real_charged_in_yuan"));
                    agent.setChargeDayOnDayPercent(detailJson.getString("charge_day_on_day_percent"));
                    agent.setAdPhotoImpression(detailJson.getLong("ad_photo_impression"));
                    agent.setAdPhotoClick(detailJson.getLong("ad_photo_click"));
                    agent.setAdItemImpression(detailJson.getLong("ad_item_impression"));
                    agent.setAdItemClick(detailJson.getLong("ad_item_click"));
                    agent.setPhotoClickRatio(detailJson.getString("photo_click_ratio"));
                    agent.setItemClickRatio(detailJson.getString("item_click_ratio"));
                    agent.setChargedCampaignCount(detailJson.getLong("charged_campaign_count"));
                    agent.setProductName(detailJson.getString("product_name"));
                    agent.setCorporationName(detailJson.getString("corporation_name"));
                    agent.setFirstCostDay(detailJson.getString("first_cost_day"));
                    agent.setIndustry(detailJson.getString("industry"));
                    agent.setSecondIndustry(detailJson.getString("second_industry"));
                    agentList.add(agent);
                }
            }
            if (!Check.isNull(agentList)) {
                infoMapper.replaceBatchKuaiShouDailyAgent(agentList);
            }
            getAgentReport(token, agentId, startDate, endDate, page + 1);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 同步快手新上监控数据
     */
    @Async("calcThreadExecutor")
    public void synchronousKuaiShouMonitor(String type, Long accId) {
        log.info("同步快手新上数据:{}", accId);
        List<Map> kuaiShouGroup = new ArrayList<>();
        List<Map> kuaiShouCampaign = new ArrayList<>();
        List<Map> TouTiaoCampaign = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        if (accId != null) {
            /*--------------------快手广告组-----------------------*/
            if (type.equals("0")) {
                //查询创建不满7天的快手组信息
                kuaiShouGroup = infoMapper.getKuaiShouGroup(sdf.format(new Date()), String.valueOf(accId));
            } else if (type.equals("1")) {
                //查询当日快手组信息
                kuaiShouGroup = infoMapper.getKuaiShouGroupNow(sdf.format(new Date()), String.valueOf(accId));
            } else {
                //查询全量快手组信息
                kuaiShouGroup = infoMapper.getKuaiShouGroup(null, String.valueOf(accId));
            }
            //处理快手组数据
            if (!kuaiShouGroup.isEmpty()) {
                kuaiShouGroup.stream().forEach(obj -> {
                    if (obj != null) {
                        String unitId = obj.get("unitId") == null ? null : String.valueOf(obj.get("unitId"));
                        String afterTime = obj.get("afterTime") == null ? null : (String) obj.get("afterTime");
                        String accountId = obj.get("accountId") == null ? null : String.valueOf(obj.get("accountId"));
                        String accountName = obj.get("accountName") == null ? null : (String) obj.get("accountName");
                        String userName = obj.get("userName") == null ? null : (String) obj.get("userName");
                        String groupCreatTime = obj.get("groupCreatTime") == null ? null : (String) obj.get("groupCreatTime");
                        if (unitId != null && afterTime != null) {
                            BigDecimal groupCharge = infoMapper.getGroupCharge(unitId, afterTime);
                            //大于500为有效广告组
                            String effective = "0";
                            if (groupCharge != null && groupCharge.compareTo(new BigDecimal(500)) == 1) {
                                //有效
                                effective = "1";
                            }
                            String effectiveResult = infoMapper.getMonitorGroupByUnitId(unitId);
                            //判断是否存在
                            if (effectiveResult != null) {
                                if (!effectiveResult.equals(effective)) {
                                    //存在更新
                                    infoMapper.updateMonitorGroupByUnitId(unitId, effective);
                                }

                            } else {
                                //不存在插入
                                infoMapper.saveMonitorGroup(accountId, accountName, userName, unitId, groupCreatTime, "1", effective);
                            }
                        }
                    }
                });
            }


            /*---------------快手广告计划------------------*/
            if (type.equals("0")) {
                //查询创建不满7天的快手广告计划
                kuaiShouCampaign = infoMapper.getKuaiShouCampaign(sdf.format(new Date()), String.valueOf(accId));
            } else if (type.equals("1")) {
                //查询当日快手广告计划
                kuaiShouCampaign = infoMapper.getKuaiShouCampaignNow(sdf.format(new Date()), String.valueOf(accId));
            } else {
                //查询全量快手广告计划
                kuaiShouCampaign = infoMapper.getKuaiShouCampaign(null, String.valueOf(accId));
            }
            //处理快手广告计划数据
            if (!kuaiShouCampaign.isEmpty()) {
                kuaiShouCampaign.stream().forEach(obj -> {
                    if (obj != null) {
                        String campaignId = obj.get("campaignId") == null ? null : String.valueOf(obj.get("campaignId"));
                        String afterTime = obj.get("afterTime") == null ? null : (String) obj.get("afterTime");
                        String accountId = obj.get("accountId") == null ? null : String.valueOf(obj.get("accountId"));
                        String accountName = obj.get("accountName") == null ? null : (String) obj.get("accountName");
                        String userName = obj.get("userName") == null ? null : (String) obj.get("userName");
                        String campaignCreatTime = obj.get("campaignCreatTime") == null ? null : (String) obj.get("campaignCreatTime");
                        if (campaignId != null && afterTime != null) {
                            BigDecimal campaignCharge = infoMapper.getCampaignCharge(campaignId, afterTime);
                            //大于500为有效广告计划
                            String effective = "0";
                            if (campaignCharge != null && campaignCharge.compareTo(new BigDecimal(500)) == 1) {
                                //有效
                                effective = "1";
                            }
                            String effectiveResult = infoMapper.getMonitorPlanByCampaignId(campaignId);
                            //判断是否存在
                            if (effectiveResult != null) {
                                if (!effectiveResult.equals(effective)) {
                                    //存在更新
                                    infoMapper.updateMonitorPlanByCampaignId(campaignId, effective);
                                }
                            } else {
                                //不存在插入
                                infoMapper.saveMonitorCampaign(accountId, accountName, userName, campaignId, campaignCreatTime, "1", effective);
                            }
                        }
                    }
                });
            }
        }
    }


    /**
     * 同步头条新上监控数据
     */
    public void synchronousTouTiaoMonitor(String type, Long accId) {
        log.info("同步头条新上数据:{}", accId);
        List<Map> kuaiShouGroup = new ArrayList<>();
        List<Map> kuaiShouCampaign = new ArrayList<>();
        List<Map> TouTiaoCampaign = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        if (accId != null) {
            /*---------------头条广告计划------------------*/
            if (type.equals("0")) {
                //查询创建不满7天的头条广告计划
                TouTiaoCampaign = infoMapper.getTouTiaoCampaign(sdf.format(new Date()), String.valueOf(accId));
            } else if (type.equals("1")) {
                //查询当日头条广告计划
                TouTiaoCampaign = infoMapper.getTouTiaoCampaignNow(sdf.format(new Date()), String.valueOf(accId));
            } else {
                //查询全量头条广告计划
                TouTiaoCampaign = infoMapper.getTouTiaoCampaign(null, String.valueOf(accId));
            }
            //处理快手广告计划数据
            if (!TouTiaoCampaign.isEmpty()) {
                TouTiaoCampaign.stream().forEach(obj -> {
                    if (obj != null) {
                        String campaignId = obj.get("campaignId") == null ? null : String.valueOf(obj.get("campaignId"));
                        String afterTime = obj.get("afterTime") == null ? null : (String) obj.get("afterTime");
                        String accountId = obj.get("accountId") == null ? null : String.valueOf(obj.get("accountId"));
                        String accountName = obj.get("accountName") == null ? null : (String) obj.get("accountName");
                        String userName = obj.get("userName") == null ? null : (String) obj.get("userName");
                        String campaignCreatTime = obj.get("campaignCreatTime") == null ? null : (String) obj.get("campaignCreatTime");
                        if (campaignId != null && afterTime != null) {
                            BigDecimal campaignCharge = infoMapper.getCampaignChargeTouTiao(campaignId, afterTime);
                            //大于500为有效广告计划
                            String effective = "0";
                            if (campaignCharge != null && campaignCharge.compareTo(new BigDecimal(500)) == 1) {
                                //有效
                                effective = "1";
                            }
                            String effectiveResult = infoMapper.getMonitorPlanByCampaignId(campaignId);
                            //判断是否存在
                            if (effectiveResult != null) {
                                if (!effectiveResult.equals(effective)) {
                                    //存在更新
                                    infoMapper.updateMonitorPlanByCampaignId(campaignId, effective);
                                }
                            } else {
                                //不存在插入
                                infoMapper.saveMonitorCampaign(accountId, accountName, userName, campaignId, campaignCreatTime, "0", effective);
                            }
                        }
                    }
                });
            }
        }
    }


    @Override
    public JSONObject getAccountReportBy(CtopOauthToken token, Long advertiserId) {
        JSONObject result = new JSONObject();
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", token.getAccessToken());

        String now = DateUtils.getNowDate("yyyy-MM-dd");
        JSONObject params = new JSONObject();
        params.put("advertiser_id", advertiserId);
        params.put("start_date", now);
        params.put("end_date", now);
        JSONObject jsonObject = JSONObject.parseObject(HttpUtils.httpPostRequest("https://ad.e.kuaishou.com/rest/openapi/v1/report/account_report", params, headers));
        if (jsonObject.getInteger("code") != 0) {
            XxlJobHelper.log("账户{}获取实时数据失败---{}", advertiserId, jsonObject.getString("message"));
        } else {
            if (!jsonObject.getJSONObject("data").getJSONArray("details").isEmpty()) {
                result = (JSONObject) jsonObject.getJSONObject("data").getJSONArray("details").get(0);
            }
        }
        return result;
    }

    @Override
    public JSONObject getAccountFundBy(CtopOauthToken token, Long advertiserId) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", "application/json");
        headers.put("Access-Token", token.getAccessToken());
        JSONObject params = new JSONObject();
        params.put("advertiser_id", advertiserId);
        JSONObject jsonObject = JSONObject.parseObject(HttpUtils.httpGetRequest("https://ad.e.kuaishou.com/rest/openapi/v1/advertiser/fund/get", headers, params));
        if (jsonObject.getInteger("code") != 0) {
            XxlJobHelper.log("查询账户余额失败==》accountId:{},message:{}", token.getAccountId(), jsonObject.getString("message"));
            return null;
        }
        return jsonObject.getJSONObject("data");
    }

    private static ExecutorService executorService = Executors.newFixedThreadPool(5);

    /**
     * 获取广告组
     */
    @Override
    public void getGroupListByAccountId(String accessToken, Long accountId) {
        List<KuaiShouCampaign> list = infoMapper.getCampaignByAccountId(accountId);
        if (Check.isNull(list)) {
            return;
        }
        for (KuaiShouCampaign campaign : list) {
            executorService.submit(new Runnable() {
                @Override
                public void run() {
                    getGroupListByPage2(accessToken, accountId, campaign.getCampaignId(), 1);
                }
            });
        }
    }

    public void getGroupListByPage2(String accessToken, Long accountId, Long campaignId, int page) {
        //广告组信息
        String url = "https://ad.e.kuaishou.com/rest/openapi/v1/ad_unit/list";
        JSONObject param = new JSONObject();
        param.put("advertiser_id", accountId);
        if (!Check.isNull(campaignId)) {
            param.put("campaign_id", campaignId);
        }
        param.put("page_size", 500);
        param.put("page", page);

        Map<String, String> headers = new HashMap<>();
        headers.put("Access-Token", accessToken);
        headers.put("Content-Type", "application/json");

        String result = HttpUtils.kuaiShouhttpPostRequest(url, param.toJSONString(), headers);
        JSONObject resultJson = JSONObject.parseObject(result);
        if (Check.isNull(resultJson)) {
            log.error("获取广告组接口异常,advertiserId:{}", accountId);
            return;
        }
        Integer code = resultJson.getInteger("code");
        if (null == code || code != 0) {
            log.error("获取广告组返回结果异常,advertiserId:{},异常信息:{}", accountId, resultJson);
            return;
        }
        JSONArray details = resultJson.getJSONObject("data").getJSONArray("details");
        if (Check.isNull(details)) {
            return;
        }
        addGroupV2(accountId, details);
        getGroupListByPage2(accessToken, accountId, campaignId, page + 1);
    }

    @Override
    public KuaiShouGroup getGroupOneByUnitId(Long unitId) {
        return infoMapper.getGroupOneByUnitId(unitId);
    }

    @Override
    public KuaiShouGroupTarget getTargetOneByUnitId(Long unitId) {
        return infoMapper.getTargetOneByUnitId(unitId);
    }

    @Override
    public List<String> getAllAccountId() {
        return infoMapper.getAllAccountId();
    }

    @Override
    public List<String> getAccountIdByMedia(String mediaId) {
        return infoMapper.getAccountIdByMedia(mediaId);
    }
}
