package cn.com.ctop.job.kuaishou.service.impl;

import cn.com.ctop.job.kuaishou.entity.KuaishouSupplyChain;
import cn.com.ctop.job.kuaishou.mapper.RuixuanMapper;
import cn.com.ctop.job.kuaishou.mapper.UserMapper;
import cn.com.ctop.job.kuaishou.service.ISendMessageService;
import cn.com.ctop.job.kuaishou.service.RuixuanService;
import cn.com.ctop.job.kuaishou.utils.Check;
import cn.com.ctop.job.kuaishou.utils.DateUtils;
import cn.com.ctop.job.kuaishou.utils.HttpUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class RuixuanServiceImpl implements RuixuanService {

    @Autowired
    private RuixuanMapper ruixuanMapper;
    @Autowired
    private ISendMessageService sendMessageService;

    @Override
    public List<JSONObject> getCookie() {
        return ruixuanMapper.getCookie();
    }

    @Override
    public Map<Long, Long> getItemMap() {
        Map<Long, Long> returnMap = new HashMap<>();
        List<JSONObject> list = ruixuanMapper.itemList();
        if (!Check.isNull(list)) {
            for (int i = 0; i < list.size(); i++) {
                JSONObject jsonObject = list.get(i);
                returnMap.put(jsonObject.getLong("itemId"), jsonObject.getLong("itemCommissionRate"));
            }
        }
        return returnMap;
    }


    @Transactional
    @Override
    public void getOrderList(String cookie, Integer offset, Long orderCreateTimeStart, Long orderCreateTimeEnd, Map<Long, Long> itemMap) throws Exception {
        String url = "https://cps.kwaixiaodian.com/distribute/pc/investment/order/list";
        Thread.sleep(1000);
        log.info("567团长抓取订单数据入参，开始时间：{},结束时间：{},数据：{}", DateUtils.tempToLongDateStr(orderCreateTimeStart), DateUtils.tempToLongDateStr(orderCreateTimeEnd), offset);
        Map<String, Object> params = new HashMap<>();
        if (!Check.isNull(orderCreateTimeStart)) {
            params.put("orderCreateTimeStart", orderCreateTimeStart);
        }
        if (!Check.isNull(orderCreateTimeEnd)) {
            params.put("orderCreateTimeEnd", orderCreateTimeEnd);
        }
        params.put("fundType", 1);
        params.put("timeType", 1);
        params.put("limit", 100);
        params.put("offset", offset);

        Map<String, String> headers = new HashMap<>();
        headers.put("Cookie", cookie);

        String resultStr = HttpUtils.KuaiShouttpGetRequest(url, params, headers);
        JSONObject result = JSONObject.parseObject(resultStr);
        if (Check.isNull(result)) {
            log.error("快分销数据返回为空");
            return;
        }
        Integer code = result.getInteger("result");
        if (code != 1) {
            log.info("快分销数据返回异常：", resultStr);
            sendMessageService.sendMessage("113dee46c7df464da78c07a985e92cd1", "快手供应链cookie失效，请及时更新。媒体返回信息：" + resultStr);
            return;
        }
        JSONObject dataJson = result.getJSONObject("data");
        if (Check.isNull(dataJson)) {
            log.error("快分销数据返回为data数据为空");
            return;
        }
        JSONArray list = dataJson.getJSONArray("list");
        if (Check.isNull(list)) {
            log.error("快分销数据返回为list数据为空");
            return;
        }
        log.info("快分销当前数据条数:{}", list.size());
        List<KuaishouSupplyChain> supplyChains = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            JSONObject jsonObject = list.getJSONObject(i);
            KuaishouSupplyChain supplyChain = new KuaishouSupplyChain();
            supplyChain.setOid(jsonObject.getLong("oid"));
            JSONObject item = jsonObject.getJSONArray("itemList").getJSONObject(0);
            if (!Check.isNull(item)) {
                supplyChain.setItemId(item.getLong("itemId"));
                supplyChain.setItemTitle(item.getString("itemTitle"));
                supplyChain.setReservePrice(item.getLong("reservePrice"));
                supplyChain.setImageUrl(item.getString("imageUrl"));
            }
            supplyChain.setOrderStatus(jsonObject.getInteger("orderStatus"));
            supplyChain.setOrderCreateTime(jsonObject.getLong("orderCreateTime"));
            supplyChain.setSendStatus(jsonObject.getInteger("sendStatus"));
            supplyChain.setSettlementBizType(jsonObject.getInteger("settlementBizType"));
            supplyChain.setSettlementBizTypeDesc(jsonObject.getString("settlementBizTypeDesc"));
            supplyChain.setSendTime(jsonObject.getLong("sendTime"));
            supplyChain.setRecvTime(jsonObject.getLong("recvTime"));
            supplyChain.setSellerId(jsonObject.getLong("sellerId"));
            supplyChain.setPromoterId(jsonObject.getLong("promoterId"));
            supplyChain.setPromoterNickName(filterSupplement(jsonObject.getString("promoterNickName")));
            supplyChain.setActivityId(jsonObject.getLong("activityId"));
            supplyChain.setOrderAmount(jsonObject.getLong("orderAmount"));
            supplyChain.setPayAmount(jsonObject.getLong("payAmount"));
            supplyChain.setRegimentalPromotionRate(jsonObject.getLong("regimentalPromotionRate"));
            supplyChain.setShareRate(jsonObject.getLong("shareRate"));
            supplyChain.setBaseAmount(jsonObject.getLong("baseAmount"));
            supplyChain.setServiceAmount(jsonObject.getLong("serviceAmount"));
            supplyChain.setRegimentalPromotionAmount(jsonObject.getLong("regimentalPromotionAmount"));
            supplyChain.setTotalRegimentalSettleAmount(jsonObject.getLong("totalRegimentalSettleAmount"));
            supplyChain.setStatDate(DateUtils.tempToDate(jsonObject.getLong("orderCreateTime")));
            supplyChain.setItemCommissionRate(itemMap.get(item.getLong("itemId")));
            supplyChains.add(supplyChain);
        }
        try {
            ruixuanMapper.replaceSupplyChains(supplyChains);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (list.size() >= 100) {
            getOrderList(cookie, offset + 100, orderCreateTimeStart, orderCreateTimeEnd, itemMap);
        }
    }

    public static String filterSupplement(String content) {
        StringBuilder sb = new StringBuilder();
        for (char ch : content.toCharArray()) {
            if (!Character.isHighSurrogate(ch) && !Character.isLowSurrogate(ch)) {
                sb.append(ch);
            }
        }
        return sb.toString();
    }
    @Override
    public List<JSONObject> getSamplesData(String startDate) {
        return ruixuanMapper.getSamplesData(startDate);
    }

    @Override
    public List<JSONObject> getDataList(Long statDate) {
        return ruixuanMapper.getDataList(statDate);
    }

    @Override
    public void deletePromoterBindChannelByDate(Long date) {
        ruixuanMapper.deletePromoterBindChannelByDate(date);
    }

    @Override
    public void cleanKuaiShouPromoterBindChannel(Long date) {
        ruixuanMapper.cleanKuaiShouPromoterBindChannel(date);

    }

    @Override
    public void cleanBytedancePromoterBindChannel(Long date) {
        ruixuanMapper.cleanBytedancePromoterBindChannel(date);

    }

    @Override
    public void deleteMgsPromoterBindChannelByDate(Long date) {
        ruixuanMapper.deleteMgsPromoterBindChannelByDate(date);

    }

    @Override
    public void cleanMgsKuaiShouPromoterBindChannel(Long date) {
        ruixuanMapper.cleanMgsKuaiShouPromoterBindChannel(date);

    }


}
