package cn.com.ctop.job.kuaishou.service.impl;

import cn.com.ctop.job.kuaishou.entity.CtopOauthToken;
import cn.com.ctop.job.kuaishou.entity.WarningOperationLog;
import cn.com.ctop.job.kuaishou.entity.WexinConfig;
import cn.com.ctop.job.kuaishou.mapper.SendMapper;
import cn.com.ctop.job.kuaishou.service.AccountInfoService;
import cn.com.ctop.job.kuaishou.service.CommonService;
import cn.com.ctop.job.kuaishou.service.IMessageTemplate;
import cn.com.ctop.job.kuaishou.service.ISendMessageService;
import cn.com.ctop.job.kuaishou.service.TokenService;
import cn.com.ctop.job.kuaishou.service.UserService;
import cn.com.ctop.job.kuaishou.utils.Check;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;

@Slf4j
@Service
public class SendMessageServiceImpl implements ISendMessageService {
    @Autowired
    private SendMapper sendMapper;

    @Autowired
    private WexinServiceImpl wexinService;

    @Autowired
    private CommonService commonService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private UserService userService;

    @Autowired
    private AccountInfoService accountInfoService;

    @Autowired
    private IMessageTemplate messageTemplate;

    @Override
    public List<WexinConfig> orderByDescList() {
        return sendMapper.orderByDescList();
    }

    @Override
    public void sendMessage(String userId, String text) {
        JSONObject weChatId = sendMapper.getWChatIdByUserId(userId);
        if (!Check.isNull(weChatId)) {
            wexinService.sendMessageByWeChatId(weChatId, text);
            log.info("消息发送成功，userId:{}", userId);
        }
    }

    /*
    快手出价方式预警 && 操作
     */
    @Override
    public void kuaiShouWarningOperation(Long projectId, String projectName, Long accountId, Long unitId, String unit_name) {
        String userId = sendMapper.queryUserIdByAccountId(accountId);
        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        accountInfoService.updateUnitStatus(token.getAccessToken(), accountId, unitId, 2, "admin");
        WarningOperationLog warningOperationLog = new WarningOperationLog();
        warningOperationLog.setAccountId(accountId);
        warningOperationLog.setSubject("出价方式预警");
        warningOperationLog.setUnitId(unitId);
        warningOperationLog.setUserId(userId);
        warningOperationLog.setWarningType(1);
        sendMapper.insertWarningOperationLog(warningOperationLog);
        try {
            String text = messageTemplate.getKuaiShouBidTypeMessage(projectName, accountId, unitId, unit_name);
            this.sendMessage(userId, text);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 出价 过高预警 && 操作
     */
    @Override
    public void kuaiShouBidWarningOperation(Long projectId, String projectName, Long accountId, Long unitId, String unit_name) {
        String userId = sendMapper.queryUserIdByAccountId(accountId);
        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        accountInfoService.updateUnitStatus(token.getAccessToken(), accountId, unitId, 2, "admin");
        WarningOperationLog warningOperationLog = new WarningOperationLog();
        warningOperationLog.setAccountId(accountId);
        warningOperationLog.setSubject("出价过高预警");
        warningOperationLog.setUnitId(unitId);
        warningOperationLog.setUserId(userId);
        warningOperationLog.setWarningType(2);
        sendMapper.insertWarningOperationLog(warningOperationLog);
        try {
            String text = messageTemplate.getKuaiShouBidMessage(projectName, accountId, unitId, unit_name);
            this.sendMessage(userId, text);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void sendYesterdayChargeEarlyWarningMessageMessage(Long accountId, String authName, Integer maxHour, BigDecimal nowCost, Long projectId, String projectName, String userName) {
        StringBuilder text = new StringBuilder();
        text.append("较昨日同时段消耗预警").append("<br/>")
                .append("您的项目：" + projectName + "下的，").append("<br/>")
                .append("账号：").append(accountId + "，").append("授权名称为：" + authName).append("<br/>")
                .append("账户所属人:" + userName).append("<br/>")
                .append("相较于昨日 " + maxHour + "时").append("<br/>")
                .append("出现消耗暴涨，").append("消耗金额为：" + nowCost).append("<br/>")
                .append("请您及时查看！");
        List<JSONObject> wexinIds = sendMapper.selectWeiXinIdListByAccountId(accountId);
        wexinService.sendMessage(wexinIds, text.toString());
    }

    public void sendChargeEarlyWarningMessage(Long accountId, String authName, Integer hour, BigDecimal amount, Long projectId, String projectName, String userName) {
        StringBuilder text = new StringBuilder();
        text.append("较上时段消耗预警").append("<br/>")
                .append("您的项目：" + projectName + " 下的").append("<br/>")
                .append("账号：").append(accountId + "，").append("授权名称为：" + authName).append("<br/>")
                .append("账户所属人:" + userName).append("<br/>")
                .append(hour + " 时").append("相较于 " + (hour - 1) + " 时").append("<br/>")
                .append("出现消耗暴涨，").append("消耗金额为：" + amount).append("<br/>")
                .append("请您及时查看！");
        List<JSONObject> wexinIds = sendMapper.selectWeiXinIdListByAccountId(accountId);
        wexinService.sendMessage(wexinIds, text.toString());
    }

}
