package cn.com.ctop.job.kuaishou.service.impl;

import cn.com.ctop.job.kuaishou.entity.CtopOauthToken;
import cn.com.ctop.job.kuaishou.entity.OauthAgentToken;
import cn.com.ctop.job.kuaishou.mapper.TokenMapper;
import cn.com.ctop.job.kuaishou.service.TokenService;
import cn.com.ctop.job.kuaishou.utils.Check;
import cn.com.ctop.job.kuaishou.utils.HttpUtils;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xxl.job.core.context.XxlJobHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
@Service
public class TokenServiceImpl implements TokenService {
    @Autowired
    private TokenMapper tokenMapper;

    @Override
    public List<CtopOauthToken> getAllKuaiShouTokens() {
        return tokenMapper.getAllKuaiShouTokens();
    }

    private static ExecutorService kuaishouTokenExecutorService = Executors.newFixedThreadPool(4);

    @Override
    public CtopOauthToken getTokenByAccountId(Long accountId) {
        CtopOauthToken one = tokenMapper.getTokenByAccountId(accountId);
        if ("2".equals(one.getMediaId()) || "4".equals(one.getMediaId())) {
            OauthAgentToken tokenByAppId = tokenMapper.getAgentTokenByAppId(one.getAppId());
            if (!Check.isNull(tokenByAppId)) {
                one.setAccessToken(tokenByAppId.getAccessToken());
            }
        }
        return one;
    }


    @Override
    public void getKuaiShouAgentRefresh() {
        List<OauthAgentToken> list = tokenMapper.getAgentList();
        if (!Check.isNull(list)) {
            for (OauthAgentToken agentToken : list) {
                getKuaiShouAgentRefreshToken(agentToken.getId(), agentToken.getAgentId(), agentToken.getAccessToken(), agentToken.getRefreshToken(), agentToken.getAppId(), agentToken.getSecret());
            }
        }
    }

    public void getKuaiShouAgentRefreshToken(String id, Long agentId, String accessToken, String refreshToken, Long appId, String secret) {
        log.info("快手刷新token入参，token:{},refreshToken:{}", accessToken, refreshToken);
        String url = "https://ad.e.kuaishou.com/rest/openapi/oauth2/authorize/refresh_token";
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", " application/json");
        Map<String, Object> param = new HashMap<>();
        param.put("app_id", appId);
        param.put("secret", secret);
        param.put("refresh_token", refreshToken);
        try {
            String result = HttpUtils.httpPostRequest(url, param, headers);
            JSONObject resultJson = JSONObject.parseObject(result);
            log.info("快手刷新代理商token返回结果：{}", refreshToken);
            if (Check.isNull(resultJson)) {
                log.error("快手刷新代理商token返回结果为空,token：{}", accessToken);
                return;
            }
            Integer code = resultJson.getInteger("code");
            if (null == code || code != 0) {
                log.error("快手刷新代理商token失败,token:{},result:{}", accessToken, resultJson);
                return;
            }
            JSONObject dataJson = resultJson.getJSONObject("data");
            if (null == dataJson) {
                log.error("快手刷新代理商token失败,token:{},result:{}", accessToken, resultJson);
                return;
            }

            OauthAgentToken updateAgentToken = new OauthAgentToken();
            updateAgentToken.setId(id);
            updateAgentToken.setAccessToken(dataJson.getString("access_token"));
            updateAgentToken.setRefreshToken(dataJson.getString("refresh_token"));
            long accessTokenExpireIn = dataJson.getLong("access_token_expires_in") * 1000L;
            long now = System.currentTimeMillis();
            Date accessTokenExpireInDate = new Date(now + accessTokenExpireIn);
            updateAgentToken.setAccessTokenExpiresIn(accessTokenExpireInDate);
            long refreshTokenExpireIn = dataJson.getLong("refresh_token_expires_in") * 1000L;
            Date refreshTokenExpireInDate = new Date(now + refreshTokenExpireIn);
            updateAgentToken.setRefreshTokenExpiresIn(refreshTokenExpireInDate);
            updateAgentToken.setUpdateTime(new Date());
            int update = tokenMapper.updateAgentTokenById(updateAgentToken);
            if (update > 0) {
                log.info("刷新代理商token完成");
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            log.error("快手刷新代理商token失败,token{}", accessToken);
        }
    }

    @Override
    public void getLiveKuaiShouAgentRefresh() {
        List<OauthAgentToken> list = tokenMapper.getLiveKuaiShouAgentList();
        if (!Check.isNull(list)) {
            for (OauthAgentToken agentToken : list) {
                getLiveKuaiShouAgentRefreshToken(agentToken.getId(), agentToken.getAccessToken(), agentToken.getRefreshToken(), agentToken.getAppId(), agentToken.getSecret());
            }
        }
    }

    public void getLiveKuaiShouAgentRefreshToken(String id, String accessToken, String refreshToken, Long appId, String secret) {
        log.info("快手刷新token入参，token:{},refreshToken:{}", accessToken, refreshToken);
        String url = "https://ad.e.kuaishou.com/rest/openapi/oauth2/authorize/refresh_token";
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", " application/json");
        Map<String, Object> param = new HashMap<>();

        param.put("app_id", appId);
        param.put("secret", secret);
        param.put("refresh_token", refreshToken);
        try {
            String result = HttpUtils.httpPostRequest(url, param, headers);
            JSONObject resultJson = JSONObject.parseObject(result);
            log.info("快手刷新代理商token返回结果：{}", refreshToken);
            if (Check.isNull(resultJson)) {
                log.error("快手刷新代理商token返回结果为空,token：{}", accessToken);
                return;
            }
            Integer code = resultJson.getInteger("code");
            if (null == code || code != 0) {
                log.error("快手刷新代理商token失败,token:{},result:{}", accessToken, resultJson);
                return;
            }
            JSONObject dataJson = resultJson.getJSONObject("data");
            if (null == dataJson) {
                log.error("快手刷新代理商token失败,token:{},result:{}", accessToken, resultJson);
                return;
            }
            OauthAgentToken updateAgentToken = new OauthAgentToken();
            updateAgentToken.setId(id);
            updateAgentToken.setAccessToken(dataJson.getString("access_token"));
            updateAgentToken.setRefreshToken(dataJson.getString("refresh_token"));
            long accessTokenExpireIn = dataJson.getLong("access_token_expires_in") * 1000L;
            long now = System.currentTimeMillis();
            Date accessTokenExpireInDate = new Date(now + accessTokenExpireIn);
            updateAgentToken.setAccessTokenExpiresIn(accessTokenExpireInDate);
            long refreshTokenExpireIn = dataJson.getLong("refresh_token_expires_in") * 1000L;
            Date refreshTokenExpireInDate = new Date(now + refreshTokenExpireIn);
            updateAgentToken.setRefreshTokenExpiresIn(refreshTokenExpireInDate);
            updateAgentToken.setUpdateTime(new Date());
            int update = tokenMapper.updateLiveById(updateAgentToken);
            if (update > 0) {
                log.info("刷新代理商token完成");
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            log.error("快手刷新代理商token失败,token{}", accessToken);
        }
    }

    @Override
    public void getKuaiShouRefresh() {
        List<CtopOauthToken> oauthTokens = tokenMapper.getAllKuaiShouTokens();
        if (!Check.isNull(oauthTokens)) {
            Map<Long, String> configMap = getConfigMap();
            for (CtopOauthToken oauthToken : oauthTokens) {
                XxlJobHelper.log("刷新token,accountId:{}", oauthToken.getAccountId());
                //快手
                if ("2".equals(oauthToken.getMediaId().trim())) {
                    String secret = configMap.get(oauthToken.getAppId());
                    kuaishouTokenExecutorService.submit(new Runnable() {
                        @Override
                        public void run() {
                            getKuaiShouRefreshToken(oauthToken.getAccountId(), oauthToken.getAccessToken(), oauthToken.getRefreshToken(), oauthToken.getAppId(), secret);
                        }
                    });
                }
            }
        } else {
            XxlJobHelper.log("----------------------刷新token获取数据返回结果为空----------------------------------");
        }
    }


    public Map<Long, String> getConfigMap() {
        Map<Long, String> map = new HashMap<>();
        List<JSONObject> list = tokenMapper.getListByMediaId();
        if (!Check.isNull(list)) {
            for (int i = 0; i < list.size(); i++) {
                JSONObject jsonObject = list.get(i);
                map.put(jsonObject.getLong("app_id"), jsonObject.getString("secret"));
            }
        }
        return map;
    }


    public void getKuaiShouRefreshToken(Long accountId, String token, String refreshToken, Long appId, String secret) {
        log.info("快手刷新token入参，accountId:{},token:{},refreshToken:{}", accountId, token, refreshToken);
        String url = "https://ad.e.kuaishou.com/rest/openapi/oauth2/authorize/refresh_token";
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-Type", " application/json");
        Map<String, Object> param = new HashMap<>();
        param.put("app_id", appId);
        param.put("secret", secret);
        param.put("refresh_token", refreshToken);
        try {
            CtopOauthToken oauthToken = tokenMapper.getTokenByAccountId(accountId);
            if (null == oauthToken) {
                log.error("查询token信息返回结果为空,accountId：{}", accountId);
                return;
            }
            XxlJobHelper.log("刷新token,入參:{}，token:{}", param, headers);
            String result = HttpUtils.httpPostRequest(url, param, headers);
            JSONObject resultJson = JSONObject.parseObject(result);
            log.info("快手刷新token返回结果，accountId:{},返回结果：{}", accountId, resultJson);
            if (Check.isNull(resultJson)) {
                log.error("快手刷新token返回结果为空,accountId：{}", accountId);
                return;
            }
            Integer code = resultJson.getInteger("code");
            if (null == code || code != 0) {
                log.error("快手刷新token失败,accountId:{},result:{}", accountId, resultJson);
                return;
            }
            JSONObject dataJson = resultJson.getJSONObject("data");
            if (null == dataJson) {
                log.error("快手刷新token失败,accountId:{},result:{}", accountId, resultJson);
                return;
            }
            oauthToken.setAccessToken(dataJson.getString("access_token"));
            long accessTokenExpireIn = dataJson.getLong("access_token_expires_in") * 1000L;
            long now = System.currentTimeMillis();
            Date accessTokenExpireInDate = new Date(now + accessTokenExpireIn);
            oauthToken.setAccessTokenExpiresIn(accessTokenExpireInDate);
            oauthToken.setRefreshToken(dataJson.getString("refresh_token"));
            long refreshTokenExpireIn = dataJson.getLong("refresh_token_expires_in") * 1000L;
            Date refreshTokenExpireInDate = new Date(now + refreshTokenExpireIn);
            oauthToken.setRefreshTokenExpiresIn(refreshTokenExpireInDate);
            oauthToken.setUpdateTime(new Date());
            int update = tokenMapper.updateTokenById(oauthToken);
            if (update > 0) {
                log.info("快手刷新token成功，accountId:{}", accountId);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            log.error("快手刷新token失败,accountId{}", accountId);
        }
    }

    @Override
    public OauthAgentToken getAgentTokenByAppId(Long appId) {
        return tokenMapper.getAgentTokenByAppId(appId);
    }

    @Override
    public List<CtopOauthToken> queryKsValidRuleAccountsToken() {
        return tokenMapper.queryKsValidRuleAccountsToken();
    }
}
