package cn.com.ctop.job.kuaishou.service.impl;

import cn.com.ctop.job.kuaishou.entity.Project;
import cn.com.ctop.job.kuaishou.entity.UserAllocation;
import cn.com.ctop.job.kuaishou.mapper.UserMapper;
import cn.com.ctop.job.kuaishou.service.UserService;
import cn.com.ctop.job.kuaishou.utils.Check;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class UserServiceImpl implements UserService {
    @Autowired
    private UserMapper userMapper;

    @Override
    public Map<String, String> getUserMapByMap(Map<String, String> userIdMap) {

        Map<String, String> userMap = new HashMap<>();
        if (!Check.isNullMap(userIdMap)) {
            List<JSONObject> userList = userMapper.getUserMapByMap(userIdMap);
            if (!Check.isNull(userList)) {
                for (int i = 0; i < userList.size(); i++) {
                    JSONObject userJson = userList.get(i);
                    userMap.put(userJson.getString("id"), userJson.getString("realname"));
                }
            }
        }
        return userMap;
    }


    @Override
    public Map<Long, String> getProjectMap(Map<Long, Long> projectIdMap) {
        Map<Long, String> projectMap = new HashMap<>();
        if (!Check.isNullMap(projectIdMap)) {
            List<JSONObject> projectList = userMapper.getListByMap(projectIdMap);
            if (!Check.isNull(projectList)) {
                for (int i = 0; i < projectList.size(); i++) {
                    JSONObject projectJson = projectList.get(i);
                    projectMap.put(projectJson.getLong("id"), projectJson.getString("projectName"));
                }
            }
        }
        return projectMap;
    }

    @Override
    public JSONObject getBidTypeAndMaxBid(Long accountId) {
        Long projectId = userMapper.getProjectIdByAccountId(accountId);
        if (Check.isNull(projectId)) {
            log.error("此账户未绑定项目");
            return null;
        }
        return userMapper.getWarningInfoByProjectId(projectId);
    }

    @Override
    public UserAllocation getOneByAccountId(Long accountId) {
        return userMapper.getOneByAccountId(accountId);
    }

    @Override
    public Project getProjectById(Long projectId) {
        return userMapper.getProjectById(projectId);
    }

    @Override
    public List<Long> getAccountIdsByMedia(String mediaId) {
        return userMapper.getAccountIdsByMedia(mediaId);
    }


}
